package com.jz.sms.channel.dmHub.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.jz.sms.channel.chuanglan.dto.ReportDto;
import com.jz.sms.channel.dmHub.config.DmHubConfig;
import com.jz.sms.channel.dmHub.service.DmHubService;
import com.jz.sms.channel.dto.DmhubReport;
import com.jz.sms.channel.dto.ReportDetail;
import com.jz.util.HttpUtil;
import com.jz.util.OkHttpUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * Dm hub系统交互业务层
 */
@Service
public class DmHubServiceImpl implements DmHubService {

    private static final Logger log = LoggerFactory.getLogger(DmHubServiceImpl.class);

    private static final String TOKEN_KEY = "DM_HUB_API_TOKEN";

    @Autowired
    RedisTemplate redisTemplate;


    @Override
    public String getToken() {
        try {
            String token = redisTemplate.opsForValue().get(TOKEN_KEY).toString();
            Long expire = redisTemplate.opsForValue().getOperations().getExpire(TOKEN_KEY);
            if (expire != null && expire > 0L) {
                return token;
            } else {
                String appid = DmHubConfig.applicationId;
                String secret = DmHubConfig.applicationKey;
                String tokenUrl = DmHubConfig.tokenUrl;
                String grantType = "client_credentials";

                Map<String, String> requestMap = new HashMap<>();
                requestMap.put("appid", appid);
                requestMap.put("secret", secret);
                requestMap.put("grant_type", grantType);
                String retStr = OkHttpUtil.get(tokenUrl, requestMap);
                JSONObject retJson = JSONObject.parseObject(retStr);
                String errorCode = retJson.getString("error_code");
                if ("0".equals(errorCode)) {
                    String accessToken = retJson.getString("access_token");
                    String expires = retJson.getString("expires_in");
                    long exTime = Long.parseLong(expires) - 60 * 10L;
                    redisTemplate.opsForValue().set(TOKEN_KEY, accessToken, exTime, TimeUnit.SECONDS);
                    return accessToken;
                } else {
                    // TODO 系统告警，获取API接口TOKEN失败。
                    return null;
                }
            }
        } catch (Exception e) {
            log.error("获取API接口TOKEN业务逻辑异常，异常信息：\r\n", e);
            return null;
        }

    }

    @Override
    public Map<String, Object> smsReport(ReportDto dto) throws Exception {

        try {
            String token = getToken();
            if (StringUtils.isBlank(token)) {
                return null;
            }
            DmhubReport report = new DmhubReport();
            report.setAccess_token(token);
            report.setType("sms");
            ReportDetail detail = new ReportDetail();
            detail.setAudienceId(dto.getMobile());
            detail.setBatchId(dto.getUid());
            detail.setReportDate(dto.getReportTime());
            detail.setMobile(dto.getMobile());
            detail.setChargeNum(Integer.valueOf(dto.getLength()));
            detail.setSendDate(dto.getReportTime());
            if ("DELIVRD".equals(dto.getStatus())) {
                detail.setIsReceive(true);
            } else {
                detail.setIsReceive(false);
            }
            detail.setErrMessage(dto.getStatusDesc());
            report.setReports(detail);
            log.info("短信状态回执参数：{}", JSONObject.toJSONString(report));
            String retStr = HttpUtil.sendPost(DmHubConfig.report, JSONObject.toJSONString(report));
            log.info("短信状态回执响应：{}", retStr);
            //TODO 目前没有测试环境的请求地址

        } catch (Exception e) {
            log.info("发送短信回执到DMHub异常\r\n短信信息：{}，\r\n,异常：", dto, e);
        }

        return null;

    }
}
